package Sprog::Gear::Grep;

=begin sprog-gear-metadata

  title: Pattern Match
  type_in: P
  type_out: P
  keywords: grep regex regular expression search

=end sprog-gear-metadata

=cut

use strict;

use base qw(
  Sprog::Mixin::InputByLine
  Sprog::Gear
);

__PACKAGE__->declare_properties(
  pattern       => undef,
  ignore_case   => 1,
  invert_match  => 0,
);


sub engage {
  my $self = shift;

  my $pattern = $self->pattern;

  delete $self->{regex};
  if(defined($pattern) and $pattern ne '') {
    if($self->ignore_case) {
      $self->{regex} = qr/$pattern/i;
    }
    else {
      $self->{regex} = qr/$pattern/;
    }
  }

  return $self->SUPER::engage;
}


sub line {
  my($self, $line) = @_;

  if(my $regex = $self->{regex}) {
    return unless($line =~ $regex ^ ($self->{invert_match} ? 1 : 0));
  }

  $self->msg_out(data => $line);
}


sub dialog_xml {
#  return 'file:/home/grant/projects/sprog/glade/grep.glade';

  return <<'END_XML';
<?xml version="1.0" standalone="no"?> <!--*- mode: xml -*-->
<!DOCTYPE glade-interface SYSTEM "http://glade.gnome.org/glade-2.0.dtd">

<glade-interface>

<widget class="GtkDialog" id="properties">
  <property name="title" translatable="yes">Properties</property>
  <property name="type">GTK_WINDOW_TOPLEVEL</property>
  <property name="window_position">GTK_WIN_POS_MOUSE</property>
  <property name="modal">False</property>
  <property name="resizable">True</property>
  <property name="destroy_with_parent">True</property>
  <property name="decorated">True</property>
  <property name="skip_taskbar_hint">True</property>
  <property name="skip_pager_hint">True</property>
  <property name="type_hint">GDK_WINDOW_TYPE_HINT_DIALOG</property>
  <property name="gravity">GDK_GRAVITY_NORTH_WEST</property>
  <property name="has_separator">True</property>

  <child internal-child="vbox">
    <widget class="GtkVBox" id="dialog-vbox1">
      <property name="visible">True</property>
      <property name="homogeneous">False</property>
      <property name="spacing">0</property>

      <child internal-child="action_area">
	<widget class="GtkHButtonBox" id="dialog-action_area1">
	  <property name="visible">True</property>
	  <property name="layout_style">GTK_BUTTONBOX_END</property>

	  <child>
	    <widget class="GtkButton" id="cancelbutton1">
	      <property name="visible">True</property>
	      <property name="can_default">True</property>
	      <property name="can_focus">True</property>
	      <property name="label">gtk-cancel</property>
	      <property name="use_stock">True</property>
	      <property name="relief">GTK_RELIEF_NORMAL</property>
	      <property name="focus_on_click">True</property>
	      <property name="response_id">-6</property>
	    </widget>
	  </child>

	  <child>
	    <widget class="GtkButton" id="okbutton1">
	      <property name="visible">True</property>
	      <property name="can_default">True</property>
	      <property name="has_default">True</property>
	      <property name="can_focus">True</property>
	      <property name="label">gtk-ok</property>
	      <property name="use_stock">True</property>
	      <property name="relief">GTK_RELIEF_NORMAL</property>
	      <property name="focus_on_click">True</property>
	      <property name="response_id">-5</property>
	    </widget>
	  </child>
	</widget>
	<packing>
	  <property name="padding">0</property>
	  <property name="expand">False</property>
	  <property name="fill">True</property>
	  <property name="pack_type">GTK_PACK_END</property>
	</packing>
      </child>

      <child>
	<widget class="GtkTable" id="table1">
	  <property name="visible">True</property>
	  <property name="n_rows">2</property>
	  <property name="n_columns">2</property>
	  <property name="homogeneous">False</property>
	  <property name="row_spacing">0</property>
	  <property name="column_spacing">0</property>

	  <child>
	    <widget class="GtkEntry" id="PAD.pattern">
	      <property name="visible">True</property>
	      <property name="can_focus">True</property>
	      <property name="editable">True</property>
	      <property name="visibility">True</property>
	      <property name="max_length">0</property>
	      <property name="text" translatable="yes"></property>
	      <property name="has_frame">True</property>
	      <property name="invisible_char">*</property>
	      <property name="activates_default">True</property>
	    </widget>
	    <packing>
	      <property name="left_attach">1</property>
	      <property name="right_attach">2</property>
	      <property name="top_attach">0</property>
	      <property name="bottom_attach">1</property>
	      <property name="x_padding">5</property>
	      <property name="y_padding">5</property>
	      <property name="y_options"></property>
	    </packing>
	  </child>

	  <child>
	    <widget class="GtkLabel" id="">
	      <property name="visible">True</property>
	      <property name="label" translatable="yes">Pattern:</property>
	      <property name="use_underline">False</property>
	      <property name="use_markup">False</property>
	      <property name="justify">GTK_JUSTIFY_LEFT</property>
	      <property name="wrap">False</property>
	      <property name="selectable">False</property>
	      <property name="xalign">0</property>
	      <property name="yalign">0.5</property>
	      <property name="xpad">0</property>
	      <property name="ypad">0</property>
	    </widget>
	    <packing>
	      <property name="left_attach">0</property>
	      <property name="right_attach">1</property>
	      <property name="top_attach">0</property>
	      <property name="bottom_attach">1</property>
	      <property name="x_padding">5</property>
	      <property name="y_padding">5</property>
	      <property name="x_options">fill</property>
	      <property name="y_options"></property>
	    </packing>
	  </child>

	  <child>
	    <widget class="GtkVBox" id="vbox1">
	      <property name="visible">True</property>
	      <property name="homogeneous">False</property>
	      <property name="spacing">0</property>

	      <child>
		<widget class="GtkCheckButton" id="PAD.ignore_case">
		  <property name="visible">True</property>
		  <property name="can_focus">True</property>
		  <property name="label" translatable="yes">Ignore case in pattern match</property>
		  <property name="use_underline">True</property>
		  <property name="relief">GTK_RELIEF_NORMAL</property>
		  <property name="focus_on_click">True</property>
		  <property name="active">False</property>
		  <property name="inconsistent">False</property>
		  <property name="draw_indicator">True</property>
		</widget>
		<packing>
		  <property name="padding">0</property>
		  <property name="expand">False</property>
		  <property name="fill">False</property>
		</packing>
	      </child>

	      <child>
		<widget class="GtkCheckButton" id="PAD.invert_match">
		  <property name="visible">True</property>
		  <property name="can_focus">True</property>
		  <property name="label" translatable="yes">Pass all lines except matches</property>
		  <property name="use_underline">True</property>
		  <property name="relief">GTK_RELIEF_NORMAL</property>
		  <property name="focus_on_click">True</property>
		  <property name="active">False</property>
		  <property name="inconsistent">False</property>
		  <property name="draw_indicator">True</property>
		</widget>
		<packing>
		  <property name="padding">0</property>
		  <property name="expand">False</property>
		  <property name="fill">False</property>
		</packing>
	      </child>
	    </widget>
	    <packing>
	      <property name="left_attach">0</property>
	      <property name="right_attach">2</property>
	      <property name="top_attach">1</property>
	      <property name="bottom_attach">2</property>
	      <property name="x_padding">20</property>
	      <property name="x_options"></property>
	      <property name="y_options">fill</property>
	    </packing>
	  </child>
	</widget>
	<packing>
	  <property name="padding">5</property>
	  <property name="expand">True</property>
	  <property name="fill">True</property>
	</packing>
      </child>
    </widget>
  </child>
</widget>

</glade-interface>
END_XML
}

1;

__END__

=head1 NAME

Sprog::Gear::Grep - A pattern match filter

=head1 DESCRIPTION

This is a filter gear.  It uses 'pipe' connectors for both input and output,
reads a line at a time and either discards each line or passes it down to the
next gear depending on whether the line matched a pattern (regex).  The user
can choose to include or exclude lines which match the pattern and can chose to
make the matching case sensitive or insensitive.

=head1 METHODS

=head2 line ( data )

If the string in C<data> matches the configured pattern, it will be passed to
the C<data> method of the next gear.

=head2 dialog_xml ( )

Returns a Glade dialog definition, including one entry box for specifying the
pattern and checkboxes for enabling case-insensitive matches and reversing the
sense of the match.

=head1 SEE ALSO

The L<Sprog::Mixin::InputByLine> mixin class is used to convert C<data> messages
to calls to the C<line> method.

All other methods are inherited from L<Sprog::Gear>.

=head1 COPYRIGHT 

Copyright 2004-2005 Grant McLean E<lt>grantm@cpan.orgE<gt>

This library is free software; you can redistribute it and/or modify it
under the same terms as Perl itself. 


=begin :sprog-help-text

=head1 Pattern Match Gear

The 'Pattern Match' gear allows you to filter lines of text using a pattern
match.  This gear is only intended to work with text, it probably won't do
anything useful with binary data.

If a line of data matches the pattern you entered, the whole line will be
passed through, not just the bit that matches.

=head2 Properties

The pattern match gear has three properties:

=over 4

=item Pattern

Type a Perl regular expression in the entry box.  If you don't know what that
means, start with the
L<Introduction to Regular Expressions|Sprog::help::regex_intro>.

=item Ignore case in pattern match

If you check this box, differences between upper and lower case letters will
not affect pattern matching.  For example, if your pattern was 'cat', a line
containing 'Cat' would match only if this box was checked.

=item Pass all lines except matches

If you check this box, lines which match the pattern will be discarded and only
lines which do not match will be passed through.

=back

=head2 Related Topics

The L<Find and Replace|Sprog::Gear::FindReplace> gear also uses Perl regular
expressions.

=end :sprog-help-text
